package com.cib.fintech.dfp.open.sdk;

import com.cib.fintech.dfp.open.sdk.config.Configure;
import com.cib.fintech.dfp.open.sdk.config.KeyConfigure;
import com.cib.fintech.dfp.open.sdk.enums.ReqMethodEnum;
import com.cib.fintech.dfp.open.sdk.exception.SdkExType;
import com.cib.fintech.dfp.open.sdk.exception.SdkException;
import com.cib.fintech.dfp.open.sdk.service.AbstractTransService;
import com.cib.fintech.dfp.open.sdk.util.Const;
import com.cib.fintech.dfp.open.sdk.util.FileUtil;
import com.cib.fintech.dfp.open.sdk.util.SdkUtil;

import java.io.File;
import java.io.FileOutputStream;
import java.util.HashMap;
import java.util.Map;

/**
 * 开放银行平台统一接口类
 * <p>
 * 注意：需要先配置Configure类中的值，示例参见Example.java文件
 * 该SDK中代码仅供学习和参考，开发者可以自己根据接口文档进行编码实现，而不使用该SDK中的实现
 * 各字段长度、类型等请参考具体接口文档中的定义，SDK不检查这些长度，而直接发送给服务端
 * 重要：各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，否则可能会导致安全问题
 *
 * @author zengminghua
 */
public class OpenSdk {

    /**
     * 文件ID
     */
    private static final String FILE_ID = "fileId";

    /**
     * 文件特征值算法
     */
    private static final String ALGORITHM = "SHA-1";

    /**
     * 文件哈希值key
     */
    private static final String FILE_HASH_VALUE = "fileHashValue";

    /**
     * SDK请求入口
     *
     * [重要]单应用配置方式，调用此方法前，请确保已经配置了Configure类中的变量，如keyId等
     * [重要]各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，保证传入参数的安全性。
     *
     * @param reqUri     请求资源URI
     * @param reqMethod  请求方法
     * @param headParams 头部信息请求参数
     * @param urlParams  URL请求参数
     * @param bodyParams BODY请求参数
     * @return 请求结果
     */
    public static String gateway(String reqUri, ReqMethodEnum reqMethod, Map<String, String> headParams,
            Map<String, String> urlParams, Map<String, String> bodyParams) {
        try {
            Object result = AbstractTransService.getInstance()
                    .exec(reqUri, reqMethod, headParams, urlParams, bodyParams, null, Configure.getKeyConfigure(),
                            null);
            return (String) result;
        } catch (Exception e) {
            SdkException ex = new SdkException(SdkExType.UNKNOWN);
            return ex.toString();
        }
    }

    /**
     * SDK请求入口，form格式
     *
     * [重要]多应用配置方式，调用此方法前，请确保已经配置了Configure类中的keyConfigures
     * [重要]各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，保证传入参数的安全性。
     *
     * @param reqUri     请求资源URI
     * @param reqMethod  请求方法
     * @param headParams 头部信息请求参数
     * @param urlParams  URL请求参数
     * @param bodyParams BODY请求参数
     * @param keyId      应用ID
     * @return 请求结果
     */
    public static String gatewayWithKeyId(String reqUri, ReqMethodEnum reqMethod, Map<String, String> headParams,
            Map<String, String> urlParams, Map<String, String> bodyParams, String keyId) {
        try {
            KeyConfigure keyConfigure = Configure.getKeyConfigures().get(keyId);
            if (null == keyConfigure) {
                return new SdkException(SdkExType.KEY_CONFIGURE_ERR).toString();
            }
            Object result = AbstractTransService.getInstance()
                    .exec(reqUri, reqMethod, headParams, urlParams, bodyParams, null, keyConfigure, null);
            return (String) result;
        } catch (Exception e) {
            SdkException ex = new SdkException(SdkExType.UNKNOWN);
            return ex.toString();
        }
    }

    /**
     * SDK请求入口，json格式
     *
     * [重要]多应用配置方式，调用此方法前，请确保已经配置了Configure类中的keyConfigures
     * [重要]各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，保证传入参数的安全性。
     *
     * @param reqUri     请求资源URI
     * @param reqMethod  请求方法
     * @param headParams 头部信息请求参数
     * @param urlParams  URL请求参数
     * @param bodyJson   BODY请求参数JSON字符串
     * @param keyId      应用ID
     * @return 请求结果
     */
    public static String gatewayJsonWithKeyId(String reqUri, ReqMethodEnum reqMethod, Map<String, String> headParams,
            Map<String, String> urlParams, String bodyJson, String keyId) {
        try {
            KeyConfigure keyConfigure = Configure.getKeyConfigures().get(keyId);
            if (null == keyConfigure) {
                return new SdkException(SdkExType.KEY_CONFIGURE_ERR).toString();
            }
            if (!SdkUtil.isJSONString(bodyJson)) {
                return new SdkException(SdkExType.PARAM_ERR).toString();
            }
            Object result = AbstractTransService.getInstance()
                    .exec(reqUri, reqMethod, headParams, urlParams, null, null, keyConfigure, bodyJson);
            return (String) result;
        } catch (Exception e) {
            SdkException ex = new SdkException(SdkExType.UNKNOWN);
            return ex.toString();
        }
    }

    /**
     * SDK请求入口，form格式
     *
     * [重要]多应用配置方式，调用此方法前，请确保已经配置了Configure类中的keyConfigures
     * [重要]各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，保证传入参数的安全性。
     *
     * @param reqUri     请求资源URI
     * @param reqMethod  请求方法
     * @param headParams 头部信息请求参数
     * @param urlParams  URL请求参数
     * @param bodyParams BODY请求参数
     * @param keyId      应用ID
     * @return 请求结果
     */
    public static String gatewayNetSignWithKeyId(String reqUri, ReqMethodEnum reqMethod, Map<String, String> headParams,
                                                 Map<String, String> urlParams,Map<String, String> bodyParams, String bodyJson, String keyId) {
        try {
            KeyConfigure keyConfigure = Configure.getKeyConfigures().get(keyId);
            if (null == keyConfigure) {
                return new SdkException(SdkExType.KEY_CONFIGURE_ERR).toString();
            }
            Object result = AbstractTransService.getInstance()
                    .execNetSign(reqUri, reqMethod,null, null, bodyParams, null, keyConfigure, bodyJson);
            return (String) result;
        } catch (Exception e) {
            SdkException ex = new SdkException(SdkExType.UNKNOWN);
            return ex.toString();
        }
    }

    /**
     * SDK请求入口，文件下载
     *
     * [重要]单应用配置方式，调用此方法前，请确保已经配置了Configure类中的变量，如keyId等
     * [重要]各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，保证传入参数的安全性。
     *
     * @param fileId 文件ID
     * @return 若下载成功，返回下载的文件的字节数组(byte[]类型)
     */
    public static Object downloadFile(String fileId) {
        try {
            Map<String, String> urlParams = new HashMap<String, String>(8);
            urlParams.put(FILE_ID, fileId);

            Object result = AbstractTransService.getInstance()
                    .exec(Const.FILE_DOWNLOAD_URI, ReqMethodEnum.GET, null, urlParams, null, null,
                            Configure.getKeyConfigure(), null);
            /**
             * 将下载文件byte[]类型转为String并返回
             */
            /*String fileString = new String((byte[]) result);
            return fileString;*/
            return result;
        } catch (Exception e) {
            SdkException ex = new SdkException(SdkExType.UNKNOWN);
            return ex.toString();
        }
    }

    /**
     * SDK请求入口，文件下载
     *
     * [重要]多应用配置方式，调用此方法前，请确保已经配置了Configure类中的keyConfigures
     * [重要]各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，保证传入参数的安全性。
     *
     * @param fileId 文件ID
     * @param keyId  应用ID
     * @return 若下载成功，返回下载的文件的字节数组(byte[]类型)
     */
    public static Object downloadFileWithKeyId(String fileId, String keyId) {
        try {
            KeyConfigure keyConfigure = Configure.getKeyConfigures().get(keyId);
            if (null == keyConfigure) {
                return new SdkException(SdkExType.KEY_CONFIGURE_ERR).toString();
            }
            Map<String, String> urlParams = new HashMap<String, String>(8);
            urlParams.put(FILE_ID, fileId);
            Object result = AbstractTransService.getInstance()
                    .exec(Const.FILE_DOWNLOAD_URI, ReqMethodEnum.GET, null, urlParams, null, null, keyConfigure, null);
            /**
             * 将下载文件byte[]类型转为String并返回
             */
            /*String fileString = new String((byte[]) result);
            return fileString;*/
            return result;
        } catch (Exception e) {
            SdkException ex = new SdkException(SdkExType.UNKNOWN);
            return ex.toString();
        }
    }

    /**
     * SDK请求入口，文件下载
     *
     * [重要]单应用配置方式，调用此方法前，请确保已经配置了Configure类中的变量，如keyId等
     * [重要]各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，保证传入参数的安全性。
     *
     * @param fileId   文件ID
     * @param filePath 文件存储路径
     * @return 若下载成功，返回下载成功json提示字符串(String类型)
     */
    public static String downloadFile(String fileId, String filePath) {
        try {
            Map<String, String> urlParams = new HashMap<String, String>(8);
            urlParams.put(FILE_ID, fileId);
            Object result = AbstractTransService.getInstance()
                    .exec(Const.FILE_DOWNLOAD_URI, ReqMethodEnum.GET, null, urlParams, null, null,
                            Configure.getKeyConfigure(), null);
            if (result instanceof byte[]) {
                FileOutputStream fos = null;
                try {
                    File file = new File(filePath);
                    fos = new FileOutputStream(file);
                    fos.write((byte[]) result);
                } catch (Exception e) {
                    SdkException ex = new SdkException(SdkExType.FILE_ERROR_RESULT);
                    return ex.toString();
                } finally {
                    if (fos != null) {
                        try {
                            fos.close();
                        } catch (Exception e) {

                        }
                    }
                }
                return Const.FILE_SUCCESS_RESULT;
            } else {
                return (String) result;
            }
        } catch (Exception e) {
            SdkException ex = new SdkException(SdkExType.UNKNOWN);
            return ex.toString();
        }
    }

    /**
     * SDK请求入口，文件下载
     *
     * [重要]多应用配置方式，调用此方法前，请确保已经配置了Configure类中的keyConfigures
     * [重要]各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，保证传入参数的安全性。
     *
     * @param fileId   文件ID
     * @param filePath 文件存储路径
     * @param keyId    应用ID
     * @return 若下载成功，返回下载成功json提示字符串(String类型)
     */
    public static String downloadFileWithKeyId(String fileId, String filePath, String keyId) {
        try {
            KeyConfigure keyConfigure = Configure.getKeyConfigures().get(keyId);
            if (null == keyConfigure) {
                return new SdkException(SdkExType.KEY_CONFIGURE_ERR).toString();
            }
            Map<String, String> urlParams = new HashMap<String, String>(8);
            urlParams.put(FILE_ID, fileId);
            Object result = AbstractTransService.getInstance()
                    .exec(Const.FILE_DOWNLOAD_URI, ReqMethodEnum.GET, null, urlParams, null, null, keyConfigure, null);
            if (result instanceof byte[]) {
                FileOutputStream fos = null;
                try {
                    File file = new File(filePath);
                    fos = new FileOutputStream(file);
                    fos.write((byte[]) result);
                } catch (Exception e) {
                    SdkException ex = new SdkException(SdkExType.FILE_ERROR_RESULT);
                    return ex.toString();
                } finally {
                    if (fos != null) {
                        try {
                            fos.close();
                        } catch (Exception e) {

                        }
                    }
                }
                return Const.FILE_SUCCESS_RESULT;
            } else {
                return (String) result;
            }
        } catch (Exception e) {
            SdkException ex = new SdkException(SdkExType.UNKNOWN);
            return ex.toString();
        }
    }

    /**
     * SDK请求入口，文件上传
     *
     * [重要]单应用配置方式，调用此方法前，请确保已经配置了Configure类中的变量，如keyId等
     * [重要]各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，保证传入参数的安全性。
     * [重要]SDK不校验文件大小，开发者请根据实际情况自行限制
     *
     * @param filePath 文件路径
     * @return 文件ID
     */
    public static String uploadFile(String filePath) {
        try {
            File file = new File(filePath);
            if (!file.exists()) {
                SdkException ex = new SdkException(SdkExType.FILE_ERROR_RESULT);
                return ex.toString();
            }
            // 计算文件的哈希值
            String fileHashValue = FileUtil.fileHash(filePath, ALGORITHM);
            Map<String, String> bodyParams = new HashMap<String, String>(8);
            bodyParams.put(FILE_HASH_VALUE, fileHashValue);
            Object result = AbstractTransService.getInstance()
                    .exec(Const.FILE_UPLOAD_URI, ReqMethodEnum.POST, null, null, bodyParams, filePath,
                            Configure.getKeyConfigure(), null);
            return (String) result;
        } catch (Exception e) {
            SdkException ex = new SdkException(SdkExType.UNKNOWN);
            return ex.toString();
        }
    }

    /**
     * SDK请求入口，文件上传
     *
     * [重要]多应用配置方式，调用此方法前，请确保已经配置了Configure类中的keyConfigures
     * [重要]各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，保证传入参数的安全性。
     * [重要]SDK不校验文件大小，开发者请根据实际情况自行限制
     *
     * @param filePath 文件路径
     * @param keyId    应用ID
     * @return 文件ID
     */
    public static String uploadFileWithKeyId(String filePath, String keyId) {
        try {
            KeyConfigure keyConfigure = Configure.getKeyConfigures().get(keyId);
            if (null == keyConfigure) {
                return new SdkException(SdkExType.KEY_CONFIGURE_ERR).toString();
            }
            File file = new File(filePath);
            if (!file.exists()) {
                SdkException ex = new SdkException(SdkExType.FILE_ERROR_RESULT);
                return ex.toString();
            }
            // 计算文件的哈希值
            String fileHashValue = FileUtil.fileHash(filePath, ALGORITHM);
            Map<String, String> bodyParams = new HashMap<String, String>(8);
            bodyParams.put(FILE_HASH_VALUE, fileHashValue);
            Object result = AbstractTransService.getInstance()
                    .exec(Const.FILE_UPLOAD_URI, ReqMethodEnum.POST, null, null, bodyParams, filePath, keyConfigure,
                            null);
            return (String) result;
        } catch (Exception e) {
            SdkException ex = new SdkException(SdkExType.UNKNOWN);
            return ex.toString();
        }
    }
}