package com.cib.fintech.dfp.open.sdk.util;


import com.cib.fintech.dfp.open.sdk.exception.SdkExType;
import com.cib.fintech.dfp.open.sdk.exception.SdkException;

import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

/**
 * 回调相关工具类
 *
 * @author wenjunxin
 * @version 4.4.0
 */
public class CallbackUtil {

    /**
     * 对回调请求进行解密验签
     *
     * @param keyId      应用ID（请求头中获得，字段名：Keyid，示例：KYONLYFORTEST012345678)
     * @param timestamp  时间戳（请求头中获得，字段名：Timestamp，示例：20210810185848 其中时间格式为yyyyMMddHHmmss）
     * @param nonce      随机数（请求头中获得，字段名：Nonce，示例：CGYHt6YaCn7aFm8B76yXTW1Q）
     * @param signValue  签名值（请求头中获得，字段名：Signature，示例：T3J8o4zY5eGvG4to7xkGfqTZN3Nk4w8NSKG/ADdxhFy+OhhebHUS8t4Wk+w3HSA4K2UXJA5o8AUXm/QAa98FFg==）
     * @param body       回调请求数据（请求体中获得，参数名：ciphertext，加密内容，示例：{"ciphertext":"xq1hzkLPC4JQC6FVE7nCzroDjWK+3WtYGrYuwpgGsX4="} ）
     * @param decryptKey 解密秘钥（sm4加密，示例：iij8IGO+6+ZVpRshrug3IQ==）
     * @param publicKey  验签公钥（sm2加密，示例：BCfPH6Alxb18eLXup8zF4nw3IZrf44FOZYSIlEaI1ePmWijbrBHiEGSWZzMcBbbcytRoRFLftlp/g45Y/m2b8zE=）
     * @return 解密验签后的body（sm4解密sm2签名，Json字符串，解密后示例：{xxx:xxx,xxx:xxx}）
     */
    public static String decryptAndVerify(String keyId, String timestamp, String nonce, String signValue, String body,
            String decryptKey, String publicKey) throws SdkException {

        //解密后的body
        String decryptedBody;
        // 解密
        try {
            decryptedBody = SymmetricEncrypt.sm4Decrypt(body, decryptKey);
        } catch (Exception e) {
            throw new SdkException(SdkExType.CALLBACK_BODY_ENCRYPT_ERR);
        }

        // 验签
        if (SdkUtil.notBlank(signValue)) {
            StringBuilder signParams = new StringBuilder().append(keyId).append("&").append(timestamp).append("&")
                    .append(nonce).append("&").append(decryptedBody);
            try {
                byte[] bytes = signParams.toString().getBytes(Const.CHARSET);
                if (!VerifyRespSignature.verifyBySM2(bytes, signValue, publicKey)) {
                    throw new SdkException(SdkExType.CALLBACK_REQUEST_VERIFY_ERR);
                }
            } catch (Exception e) {
                throw new SdkException(SdkExType.CALLBACK_REQUEST_VERIFY_ERR);
            }
        } else {
            throw new SdkException(SdkExType.CALLBACK_REQUEST_VERIFY_ERR);
        }

        return decryptedBody;
    }

    /**
     * 对回调请求进行解密验签(V2版本)
     *
     * @param keyId      应用ID（请求头中获得，字段名：Keyid，示例：KYONLYFORTEST012345678)
     * @param timestamp  时间戳（请求头中获得，字段名：Timestamp，示例：20210810185848 其中时间格式为yyyyMMddHHmmss）
     * @param nonce      随机数（请求头中获得，字段名：Nonce，示例：CGYHt6YaCn7aFm8B76yXTW1Q）
     * @param pwd      对称秘钥（请求头中获得，字段名：Pwd）
     * @param signValue  签名值（请求头中获得，字段名：Signature，示例：T3J8o4zY5eGvG4to7xkGfqTZN3Nk4w8NSKG/ADdxhFy+OhhebHUS8t4Wk+w3HSA4K2UXJA5o8AUXm/QAa98FFg==）
     * @param body       回调请求数据（请求体中获得，参数名：ciphertext，加密内容，示例：{"ciphertext":"xq1hzkLPC4JQC6FVE7nCzroDjWK+3WtYGrYuwpgGsX4="} ）
     * @param priKey 解密秘钥
     * @param respPubKey  验签公钥
     * @return 解密验签后的body（sm4解密sm2签名，Json字符串，解密后示例：{xxx:xxx,xxx:xxx}）
     */
    public static Map<String, String> decryptAndVerifyV2(String keyId, String timestamp, String nonce, String pwd, String signValue, String body,
            String priKey, String respPubKey) throws SdkException {

        Map<String, String> resultMap = new HashMap<>(2);
        try {
            // 解密对称秘钥
            byte[] pwdByte = Signature.decryptBySM2PrivateKey(priKey, Base64.decode(pwd));
            assert pwdByte != null;
            pwd = new String(pwdByte, StandardCharsets.UTF_8);
            resultMap.put("pwd", pwd);
        } catch (Exception e) {
            throw new SdkException(SdkExType.CALLBACK_BODY_ENCRYPT_ERR);
        }

        //解密后的body
        String decryptedBody;
        // 解密
        try {
            decryptedBody = SymmetricEncrypt.sm4Decrypt(body, pwd);
        } catch (Exception e) {
            throw new SdkException(SdkExType.CALLBACK_BODY_ENCRYPT_ERR);
        }

        // 验签
        if (SdkUtil.notBlank(signValue)) {
            StringBuilder signParams = new StringBuilder().append(keyId).append("&").append(timestamp).append("&")
                    .append(nonce).append("&").append(pwd).append("&").append(decryptedBody);
            try {
                byte[] bytes = signParams.toString().getBytes(Const.CHARSET);
                if (!VerifyRespSignature.verifyBySM2(bytes, signValue, respPubKey)) {
                    throw new SdkException(SdkExType.CALLBACK_REQUEST_VERIFY_ERR);
                }
                resultMap.put("body", decryptedBody);
            } catch (Exception e) {
                throw new SdkException(SdkExType.CALLBACK_REQUEST_VERIFY_ERR);
            }
        } else {
            throw new SdkException(SdkExType.CALLBACK_REQUEST_VERIFY_ERR);
        }

        return resultMap;
    }

    /**
     * 回调响应签名
     *
     * @param keyId      应用ID
     * @param timestamp  时间戳
     * @param nonce      随机数
     * @param body       响应内容
     * @param privateKey 私钥
     * @return 签名值
     */
    public static String signature(String keyId, String timestamp, String nonce,
            String body, String privateKey) throws SdkException {
        StringBuilder signParams;
        if (SdkUtil.notBlank(keyId) && SdkUtil.notBlank(timestamp) &&
                SdkUtil.notBlank(nonce) && SdkUtil.notBlank(body)) {
            signParams = new StringBuilder().append(keyId).append("&").append(timestamp).append("&")
                    .append(nonce).append("&").append(body);
        } else {
            throw new SdkException(SdkExType.CALLBACK_RESPONSE_SIGN_ERR);
        }
        return Signature.signatureBySM2(signParams.toString(), privateKey, Const.CHARSET);
    }

    /**
     * 回调响应加密
     *
     * @param body       需要加密的内容
     * @param encryptKey 加密密钥
     * @return 密文（Base64 encode）
     */
    public static String encrypt(String body, String encryptKey) throws SdkException {
        String encryptedBody;
        try {
            encryptedBody = SymmetricEncrypt.sm4Encrypt(body, encryptKey);
        } catch (Exception e) {
            throw new SdkException(SdkExType.CALLBACK_BODY_ENCRYPT_ERR);
        }
        return encryptedBody;
    }
}
