package com.cib.fintech.dfp.open.sdk.util;

import com.cib.fintech.dfp.open.sdk.exception.SdkExType;
import com.cib.fintech.dfp.open.sdk.exception.SdkException;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.Key;
import java.security.SecureRandom;

/**
 * 字段加密工具类
 * <p>
 * 支持对称加密算法 AES 的加密、解密，本类涉及的byte编码均为UTF-8
 *
 * @author zengminghua
 */
public class SymmetricEncrypt {

    private static final BouncyCastleProvider BC = new BouncyCastleProvider();

    /**
     * 字段加密算法：SM4
     */
    private static final String ALGORITHM_SM4 = "SM4";

    /**
     * 密码器类型，算法/模式/补码方式
     */
    private static final String ALGORITHM_SM4_CBC_PKCS5 = "SM4/CBC/PKCS5Padding";

    /**
     * 字段加密算法：AES
     */
    private static final String ALGORITHM_AES = "AES";

    /**
     * 密码器类型，算法/模式/补码方式
     */
    private static final String ALGORITHM_AES_CBC_PKCS5 = "AES/CBC/PKCS5Padding";

    /**
     * 密钥长度
     */
    private static final int SYMMETRIC_KEY_LENGTH = 128;

    /**
     * 安全随机数算法
     */
    private static final String SECURE_RANDOM_ALGORITHMS = "SHA1PRNG";

    /**
     * 密钥长度
     */
    private static final int SM4_KEY_LENGTH = 128;

    /**
     * AES对称加密
     *
     * @param content            待加密内容
     * @param reqParamEncryptKey 字段加密密钥
     * @return result 密文（Base64 encode）
     */
    public static String aesEncrypt(String content, String reqParamEncryptKey) throws Exception {

        try {
            // 创建密码器
            Cipher cipher = Cipher.getInstance(ALGORITHM_AES_CBC_PKCS5);

            // 初始化向量值
            IvParameterSpec iv = new IvParameterSpec(initIv(ALGORITHM_AES_CBC_PKCS5));

            // 使用密钥初始化，设置为 [加密模式] 的密码器
            cipher.init(Cipher.ENCRYPT_MODE, getSecretKey(reqParamEncryptKey), iv);
            byte[] result = cipher.doFinal(content.getBytes(Const.CHARSET));

            // 通过Base64转码返回
            return Base64.encode(result);
        } catch (Exception e) {
            throw new SdkException(SdkExType.REQ_PARAM_ENCRYPT_ERR);
        }
    }

    /**
     * AES解密操作
     *
     * @param content            待解密内容
     * @param reqParamEncryptKey 字段加密密钥
     * @return result 明文
     */
    public static String aesDecrypt(String content, String reqParamEncryptKey) throws Exception {

        try {
            // 创建密码器
            Cipher cipher = Cipher.getInstance(ALGORITHM_AES_CBC_PKCS5);

            // 初始化向量值
            IvParameterSpec iv = new IvParameterSpec(initIv(ALGORITHM_AES_CBC_PKCS5));

            // 使用密钥初始化，设置为 [解密模式] 的密码器
            cipher.init(Cipher.DECRYPT_MODE, getSecretKey(reqParamEncryptKey), iv);

            byte[] result = cipher.doFinal(Base64.decode(content));
            return new String(result, Const.CHARSET);
        } catch (Exception e) {
            throw new SdkException(SdkExType.REQ_PARAM_ENCRYPT_ERR);
        }
    }

    /**
     * 生成密钥对象
     *
     * @param symmetricKey 密钥
     * @return
     */
    private static SecretKeySpec getSecretKey(String symmetricKey) throws Exception {

        // 返回生成指定算法密钥生成器的 KeyGenerator 对象
        KeyGenerator keyGenerator = null;

        try {
            // 指定为AES的KeyGenerator
            keyGenerator = KeyGenerator.getInstance(ALGORITHM_AES);

            // 指定随机数
            SecureRandom random = SecureRandom.getInstance(SECURE_RANDOM_ALGORITHMS);
            random.setSeed(symmetricKey.getBytes(Const.CHARSET));

            keyGenerator.init(SYMMETRIC_KEY_LENGTH, random);
            SecretKey secretKey = keyGenerator.generateKey();
            byte[] enCodeFormat = secretKey.getEncoded();

            // 转换为AES专用密钥
            SecretKeySpec secretKeySpec = new SecretKeySpec(enCodeFormat, ALGORITHM_AES);
            return secretKeySpec;
        } catch (Exception e) {
            throw new SdkException(SdkExType.REQ_PARAM_ENCRYPT_ERR);
        }
    }

    /**
     * 生成SM4密钥，返回Base64编码后的密钥值（长度：24）
     *
     * @param
     * @return
     */
    public static String generateSM4Key() {
        byte[] keyBytes = generateSM4KeyBytes();
        String stringKey = Base64.encode(keyBytes);
        return stringKey;
    }

    /**
     * 生成SM4密钥，返回byte数组
     *
     * @param
     * @return
     */
    public static byte[] generateSM4KeyBytes() {
        KeyGenerator keyGenerator;
        try {
            keyGenerator = KeyGenerator.getInstance(ALGORITHM_SM4, BC);
        } catch (Exception e) {
            return null;
        }

        keyGenerator.init(SM4_KEY_LENGTH, new SecureRandom());
        SecretKey secretKey = keyGenerator.generateKey();
        return secretKey.getEncoded();
    }

    /**
     * 初始向量的方法, 全部为0
     * <p>
     *
     * @param algorithm
     * @return
     */
    private static byte[] initIv(String algorithm) {

        try {
            Cipher cipher = Cipher.getInstance(algorithm);
            int blockSize = cipher.getBlockSize();
            byte[] iv = new byte[blockSize];
            for (int i = 0; i < blockSize; ++i) {
                iv[i] = 0;
            }
            return iv;
        } catch (Exception e) {

            int blockSize = 16;
            byte[] iv = new byte[blockSize];
            for (int i = 0; i < blockSize; ++i) {
                iv[i] = 0;
            }
            return iv;
        }
    }

    /**
     * 初始向量的方法, 全部为0
     *
     * @param algorithm
     * @return
     */
    private static byte[] initBCIv(String algorithm) {

        try {
            Cipher cipher = Cipher.getInstance(algorithm, BC);
            int blockSize = cipher.getBlockSize();
            byte[] iv = new byte[blockSize];
            for (int i = 0; i < blockSize; ++i) {
                iv[i] = 0;
            }
            return iv;
        } catch (Exception e) {

            int blockSize = 16;
            byte[] iv = new byte[blockSize];
            for (int i = 0; i < blockSize; ++i) {
                iv[i] = 0;
            }
            return iv;
        }
    }

    /**
     * SM4加密
     *
     * @param content            待加密内容
     * @param reqParamEncryptKey 加密密钥
     * @return 密文（Base64 encode）
     */
    public static String sm4Encrypt(String content, String reqParamEncryptKey) throws Exception {

        try {
            // 创建密码器
            Cipher cipher = Cipher.getInstance(ALGORITHM_SM4_CBC_PKCS5, BC);

            // 初始化向量值
            IvParameterSpec iv = new IvParameterSpec(initBCIv(ALGORITHM_SM4_CBC_PKCS5));
            Key sm4Key = new SecretKeySpec(Base64.decode(reqParamEncryptKey), ALGORITHM_SM4);
            // 使用密钥初始化，设置为 [加密模式] 的密码器
            cipher.init(Cipher.ENCRYPT_MODE, sm4Key, iv);

            // 加密
            byte[] result = cipher.doFinal(content.getBytes(StandardCharsets.UTF_8));

            // 通过Base64转码返回
            return Base64.encode(result);
        } catch (Exception e) {
            throw new SdkException(SdkExType.REQ_PARAM_ENCRYPT_ERR);
        }
    }

    /**
     * SM4解密操作
     *
     * @param content            待解密内容
     * @param reqParamEncryptKey 解密密钥
     * @return 明文
     */
    public static String sm4Decrypt(String content, String reqParamEncryptKey) throws Exception {

        try {
            // 创建密码器
            Cipher cipher = Cipher.getInstance(ALGORITHM_SM4_CBC_PKCS5, BC);

            // 初始化向量值
            IvParameterSpec iv = new IvParameterSpec(initBCIv(ALGORITHM_SM4_CBC_PKCS5));
            Key sm4Key = new SecretKeySpec(Base64.decode(reqParamEncryptKey), ALGORITHM_SM4);
            // 使用密钥初始化，设置为 [解密模式] 的密码器
            cipher.init(Cipher.DECRYPT_MODE, sm4Key, iv);

            // 解密
            byte[] result = cipher.doFinal(Base64.decode(content));

            // 返回明文
            return new String(result, StandardCharsets.UTF_8);
        } catch (Exception e) {
            throw new SdkException(SdkExType.REQ_PARAM_ENCRYPT_ERR);
        }
    }
}
